/*-
 * Copyright (c) 2013 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifndef _LONGPATH_H
#define _LONGPATH_H

#include "longpath_common.h"

/*
 * large file compilation environment setup
 *
 * structure taken from OpenSolaris fcntl.h
 */

/* func -> func64 */
#if !defined(_LP64) && _FILE_OFFSET_BITS == 64
# ifdef __PRAGMA_REDEFINE_EXTNAME
#  pragma redefine_extname	openl		openl64
#  pragma redefine_extname	creatl		creatl64
#  pragma redefine_extname	statl		statl64
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   pragma redefine_extname	openatl		openatl64
#   ifdef HAVE_ATTROPEN
#    pragma redefine_extname	attropenl	attropenl64
#   endif
#   pragma redefine_extname	fstatatl	fstatatl64
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   pragma redefine_extname	lstatl		lstatl64
#  endif
#  pragma redefine_extname	mkstempl	mkstempl64
#  pragma redefine_extname	mkstempsl	mkstempsl64
#  if defined(_GNU_SOURCE)
#   pragma redefine_extname	mkostempl	mkostempl64
#   pragma redefine_extname	mkostempsl	mkostempsl64
#  endif
# else /* __PRAGMA_REDEFINE_EXTNAME */
#  define	openl(p, f, ...)		openl64(p, f, ##__VA_ARGS__)
#  define	creatl(p, m)			creatl64(p, m)
#  define	statl(p, s)   			statl64(p, s)
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   define	openatl(fd, p, f, ...)		openatl64(fd, p, f, ##__VA_ARGS__)
#   ifdef HAVE_ATTROPEN
#    define	attropenl(p, ap, f, ...)	attropenl64(p, ap, f, ##__VA_ARGS__)
#   endif
#   define	fstatatl(fd, p, s, f)		fstatatl64(fd, p, s, f)
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   define	lstatl(p, s)			lstatl64(p, s)
#  endif
#  define	mkstempl(t)			mkstempl64(t)
#  define	mkstempsl(t, l)			mkstempsl64(t, l)
#  if defined(_GNU_SOURCE)
#   define	mkostempl(t, f)			mkostempl64(t, f)
#   define	mkostempsl(t, l, f)		mkostempsl64(t, l, f)
#  endif
# endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* !_LP64 && _FILE_OFFSET_BITS == 64 */

/* func64 -> func */
#if defined(_LP64) && defined(_LARGEFILE64_SOURCE)
# ifdef __PRAGMA_REDEFINE_EXTNAME
#  pragma redefine_extname	openl64		openl
#  pragma redefine_extname	creatl64	creatl
#  pragma redefine_extname	statl64		statl
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   pragma redefine_extname	openatl64	openatl
#   ifdef HAVE_ATTROPEN
#    pragma redefine_extname	attropenl64	attropenl
#   endif
#   pragma redefine_extname	fstatatl64	fstatatl
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   pragma redefine_extname	lstatl64	lstatl
#  endif
#  pragma redefine_extname	mkstempl64	mkstempl
#  pragma redefine_extname	mkstempsl64	mkstempsl
#  if defined(_GNU_SOURCE)
#   pragma redefine_extname	mkostempl64	mkostempl
#   pragma redefine_extname	mkostempsl64	mkostempsl
#  endif
# else /* __PRAGMA_REDEFINE_EXTNAME */
#  define	openl64(p, f, ...)		openl(p, f, ##__VA_ARGS__)
#  define	creatl64(p, m)			creatl(p, m)
#  define	statl64(p, s)			statl(p, s)
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   define	openatl64(fd, p, f, ...)	openatl(fd, p, f, ##__VA_ARGS__)
#   ifdef HAVE_ATTROPEN
#    define	attropenl64(p, ap, f, ...)	attropenl(p, ap, f, ##__VA_ARGS__)
#   endif
#   define	fstatatl64(fd, p, s, f)		fstatatl(fd, p, s, f)
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   define	lstatl64(p, s)			lstatl(p, s)
#  endif
#  define	mkstempl64(t)			mkstempl(t)
#  define	mkstempsl64(t, l)		mkstempsl(t, l)
#  if defined(_GNU_SOURCE)
#   define	mkostempl64(t, f)		mkostempl(t, f)
#   define	mkostempsl64(t, l, f)		mkostempsl(t, l, f)
#  endif
# endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* _LP64 && _LARGEFILE64_SOURCE */

#ifdef  __cplusplus
extern "C" {
#endif

#if defined(__STDC__)
# define FUNCTION(type, name, ...) extern type name ## l (__VA_ARGS__);
# if     defined(_LARGEFILE64_SOURCE) && !((_FILE_OFFSET_BITS == 64) && \
            !defined(__PRAGMA_REDEFINE_EXTNAME))
#  define FUNCTION64(type, name, ...) extern type name ## l64 (__VA_ARGS__);
# else
#  define FUNCTION64(type, name, ...)
# endif /* _LARGEFILE... */
# include "longpath_tbl_functions.h"
# include "longpath_tbl_functions64.h"
# undef FUNCTION
# undef FUNCTION64
#else	/* defined(__STDC__) */
# define FUNCTION(type, name, ...) extern type name ## l ();
# if     defined(_LARGEFILE64_SOURCE) && !((_FILE_OFFSET_BITS == 64) && \
            !defined(__PRAGMA_REDEFINE_EXTNAME))
#  define FUNCTION64(type, name, ...) extern type name ## l64 ();
# else
#  define FUNCTION64(type, name, ...)
# endif /* _LARGEFILE... */
# include "longpath_tbl_functions.h"
# include "longpath_tbl_functions64.h"
# undef FUNCTION
# undef FUNCTION64
#endif	/* defined(__STDC__) */

#ifdef  __cplusplus
}
#endif

#endif /* _LONGPATH_H */
