/*-
 * Copyright (c) 2013 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */

#ifdef _LONGPATH_REDEFINE_IMPL_H
#error "longpath_redefine_impl.h included twice"
#else
#define _LONGPATH_REDEFINE_IMPL_H

#if ((! defined(_LONGPATH_REDEFINE_PRE_H)) && (! defined(_LONGPATH_REDEFINE_POST_H)))
#error "longpath_redefine_impl.h should not be included directly"
#endif

#include "longpath_config.h"

/*
 * __PRAGMA_REDEFINE_EXTNAME : _first_ redefinition wins (see gcc/c-pragma.c)
 *
 * otherwise: _last_ redefinition wins
 *
 * so longpath_redefine_pre and _post include this dependend upon __PRAGMA_REDEFINE_EXTNAME
 */

/* func -> func64 */
#if !defined(_LP64) && (_FILE_OFFSET_BITS == 64)
# ifdef __PRAGMA_REDEFINE_EXTNAME
#  pragma redefine_extname	open		openl64
#  pragma redefine_extname	creat		creatl64
#  pragma redefine_extname	stat		statl64
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   pragma redefine_extname	openat		openatl64
#   ifdef HAVE_ATTROPEN
#    pragma redefine_extname	attropen	attropenl64
#   endif
#   pragma redefine_extname	fstatat		fstatatl64
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   pragma redefine_extname	lstat		lstatl64
#  endif
#  pragma redefine_extname	mkstemp		mkstempl64
#  pragma redefine_extname	mkstemps	mkstempsl64
#  if defined(_GNU_SOURCE)
#   pragma redefine_extname	mkostemp	mkostempl64
#   pragma redefine_extname	mkostemps	mkostempsl64
#  endif
# else /* __PRAGMA_REDEFINE_EXTNAME */
#  define	open(p, f, ...)			openl64(p, f, ##__VA_ARGS__)
#  define	creat(p, m)			creatl64(p, m)
#  define	stat(p, s)   			statl64(p, s)
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   define	openat(fd, p, f, ...)		openatl64(fd, p, f, ##__VA_ARGS__)
#   ifdef HAVE_ATTROPEN
#    define	attropen(p, ap, f, ...)		attropenl64(p, ap, f, ##__VA_ARGS__)
#   endif
#   define	fstatat(fd, p, s, f)		fstatatl64(fd, p, s, f)
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   define	lstat(p, s)			lstatl64(p, s)
#  endif
#  define	mkstemp(t)			mkstempl64(t)
#  define	mkstemps(t, l)			mkstempsl64(t, l)
#  if defined(_GNU_SOURCE)
#   define	mkostemp(t, f)			mkostempl64(t, f)
#   define	mkostemps(t, l, f)		mkostempsl64(t, l, f)
#  endif
# endif  /* __PRAGMA_REDEFINE_EXTNAME */
#else  /* _LP64 && _FILE_OFFSET_BITS == 64 */
# ifdef __PRAGMA_REDEFINE_EXTNAME
#  pragma redefine_extname	open		openl
#  pragma redefine_extname	creat		creatl
#  pragma redefine_extname	stat		statl
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   pragma redefine_extname	openat		openatl
#   ifdef HAVE_ATTROPEN
#    pragma redefine_extname	attropen	attropenl
#   endif
#   pragma redefine_extname	fstatat		fstatatl
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   pragma redefine_extname	lstat		lstatl
#  endif
#  pragma redefine_extname	mkstemp		mkstempl
#  pragma redefine_extname	mkstemps	mkstempsl
#  if defined(_GNU_SOURCE)
#   pragma redefine_extname	mkostemp	mkostempl
#   pragma redefine_extname	mkostemps	mkostempsl
#  endif
# else /* __PRAGMA_REDEFINE_EXTNAME */
#  define	open(p, f, ...)			openl(p, f, ##__VA_ARGS__)
#  define	creat(p, m)			creatl(p, m)
#  define	stat(p, s)   			statl(p, s)
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   define	openat(fd, p, f, ...)		openatl(fd, p, f, ##__VA_ARGS__)
#   ifdef HAVE_ATTROPEN
#    define	attropen(p, ap, f, ...)		attropenl(p, ap, f, ##__VA_ARGS__)
#   endif
#   define	fstatat(fd, p, s, f)		fstatatl(fd, p, s, f)
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   define	lstat(p, s)			lstatl(p, s)
#  endif
#  define	mkstemp(t)			mkstempl(t)
#  define	mkstemps(t, l)			mkstempsl(t, l)
#  if defined(_GNU_SOURCE)
#   define	mkostemp(t, f)			mkostempl(t, f)
#   define	mkostemps(t, l, f)		mkostempsl(t, l, f)
#  endif
# endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif /* _LP64 && _FILE_OFFSET_BITS == 64 */

/* func64 -> func */
#if defined(_LP64) && defined(_LARGEFILE64_SOURCE)
# ifdef __PRAGMA_REDEFINE_EXTNAME
#  pragma redefine_extname	open64		openl
#  pragma redefine_extname	creat64		creatl
#  pragma redefine_extname	stat64		statl
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   pragma redefine_extname	openat64	openatl
#   ifdef HAVE_ATTROPEN
#    pragma redefine_extname	attropen64	attropenl
#   endif
#   pragma redefine_extname	fstatat64	fstatatl
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   pragma redefine_extname	lstat64		lstatl
#  endif
#  pragma redefine_extname	mkstemp64	mkstempl
#  pragma redefine_extname	mkstemps64	mkstempsl
#  if defined(_GNU_SOURCE)
#   pragma redefine_extname	mkostemp64	mkostempl
#   pragma redefine_extname	mkostemps64	mkostempsl
#  endif
# else /* __PRAGMA_REDEFINE_EXTNAME */
#  define	open64(p, f, ...)		openl(p, f, ##__VA_ARGS__)
#  define	creat64(p, m)			creatl(p, m)
#  define	stat64(p, s)   			statl(p, s)
#  if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#   define	openat64(fd, p, f, ...)		openatl(fd, p, f, ##__VA_ARGS__)
#   ifdef HAVE_ATTROPEN
#    define	attropen64(p, ap, f, ...)	attropenl(p, ap, f, ##__VA_ARGS__)
#   endif
#   define	fstatat64(fd, p, s, f)		fstatatl(fd, p, s, f)
#  endif /* defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ... */
#  if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#   define	lstat64(p, s)			lstatl(p, s)
#  endif
#  define	mkstemp64(t)			mkstempl(t)
#  define	mkstemps64(t, l)		mkstempsl(t, l)
#  if defined(_GNU_SOURCE)
#   define	mkostemp64(t, f)		mkostempl(t, f)
#   define	mkostemps64(t, l, f)		mkostempsl(t, l, f)
#  endif
# endif  /* __PRAGMA_REDEFINE_EXTNAME */
#endif  /* _LP64 && _LARGEFILE64_SOURCE */


/* func- > funcl redefinitions for all which have not been done above */

#ifdef __PRAGMA_REDEFINE_EXTNAME
#pragma redefine_extname	link	linkl
#pragma redefine_extname	unlink	unlinkl
#pragma redefine_extname	chown	chownl
#pragma redefine_extname	chmod	chmodl
#pragma redefine_extname	mknod	mknodl
#pragma redefine_extname	mkfifo	mkfifol
#pragma redefine_extname	rmdir	rmdirl
#pragma redefine_extname	mkdir	mkdirl
#pragma redefine_extname	rename	renamel
#pragma redefine_extname	utimes	utimesl
#pragma redefine_extname	lutimes	lutimesl
#pragma redefine_extname	utime	utimel

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#if defined(HAVE_LINKAT)
#pragma redefine_extname	linkat		linkatl
#endif
#pragma redefine_extname	unlinkat	unlinkatl
#if defined(HAVE_SYMLINKAT)
#pragma redefine_extname	symlinkat	symlinkatl
#endif
#if defined(HAVE_READLINKAT)
#pragma redefine_extname	readlinkat	readlinkatl
#endif
#pragma redefine_extname	fchownat	fchownatl
#if defined(HAVE_MKNODAT)
#pragma redefine_extname	mknodat		mknodatl
#endif
#if defined(HAVE_MKFIFOAT)
#pragma redefine_extname	mkfifoat	mkfifoatl
#endif
#pragma redefine_extname	rmdirat		rmdiratl
#if defined(HAVE_MKDIRAT)
#pragma redefine_extname	mkdirat		mkdiratl
#endif
#pragma redefine_extname	renameat	renameatl
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
#pragma redefine_extname	futimesat	futimesatl
#pragma redefine_extname	utimensat	utimensatl
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#pragma redefine_extname	symlink		symlinkl
#pragma redefine_extname	readlink	readlinkl
#pragma redefine_extname	lchown		lchownl
#endif

/* mkstemp */
#pragma redefine_extname	mktemp		mktempl
#pragma redefine_extname	mkdtemp		mkdtempl

/* stdlib */
#pragma redefine_extname	opendir		opendirl

/* linux */
#ifdef HAVE_FGETXATTR
#pragma	redefine_extname	getxattr	getxattrl
#pragma	redefine_extname	lgetxattr	lgetxattrl
#pragma	redefine_extname	setxattr	setxattrl
#pragma	redefine_extname	lsetxattr	lsetxattrl
#endif

/* 64 */
#if !defined(_LP64)
#pragma redefine_extname	open64		openl64
#pragma redefine_extname	creat64		creatl64
#pragma redefine_extname	stat64		statl64

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#pragma redefine_extname	openat64		openatl64
#ifdef HAVE_ATTROPEN
#pragma redefine_extname	attropen64		attropenl64
#endif
#pragma redefine_extname	fstatat64		fstatatl64
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#pragma redefine_extname	lstat64		lstatl64
#endif

/* 64 mkstemp*/
#pragma redefine_extname	mkstemp64		mkstempl64
#pragma redefine_extname	mkstemps64		mkstempsl64
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
#pragma redefine_extname	mkostemp64		mkostempl64
#pragma redefine_extname	mkostemps64		mkostempsl64
#endif
#endif /* _LP64_ */
#else /* __PRAGMA_REDEFINE_EXTNAME */
#define	link(p1, p2)			linkl(p1, p2)
#define	unlink(p)			unlinkl(p)
#define	chown(p, uid, gid)		chownl(p, uid, gid)
#define	chmod(p, m)			chmodl(p, m)
#define	mknod(p, m, d)			mknodl(p, m, d)
#define	mkfifo(p, m)			mkfifol(p, m)
#define	rmdir(p)			rmdirl(p)
#define	mkdir(p, m)			mkdirl(p, m)
#define	rename(p1, p2)			renamel(p1, p2)
#define	utimes(p, tv)			utimesl(p, tv)
#define	lutimes(p, tv)			lutimesl(p, tv)
#define	utime(p, t)			utimel(p, t)

/* at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#if defined(HAVE_LINKAT)
#define	linkat(fd1, p1, fd2, p2, f)	linkatl(fd1, p1, fd2, p2, f)
#endif
#define	unlinkat(fd, p, f)		unlinkatl(fd, p, f)
#if defined(HAVE_SYMLINKAT)
#define	symlinkat(d, fd, p)		symlinkatl(d, fd, p)
#endif
#if defined(HAVE_READLINKAT)
#define	readlinkat(fd, n, b, c)		readlinkatl(fd, n, b, c)
#endif
#define	fchownat(fd, n, uid, gid, f)	fchownatl(fd, n, uid, gid, f)
#if defined(HAVE_MKNODAT)
#define	mknodat(fd, n, m, d)		mknodatl(fd, n, m, d)
#endif
#if defined(HAVE_MKFIFOAT)
#define	mkfifoat(fd, n, m)		mkfifoatl(fd, n, m)
#endif
#define	rmdirat(fd, n)			rmdiratl(fd, n)
#if defined(HAVE_MKDIRAT)
#define	mkdirat(fd, n, m)		mkdiratl(fd, n, m)
#endif
#define	renameat(fd1, p1, fd2, p2)	renameatl(fd1, p1, fd2, p2)
#endif

/* times at */
#if defined(__EXTENSIONS__) || \
        (!defined(_STRICT_STDC) && !defined(__XOPEN_OR_POSIX))
        /* || defined(_XPG7) */
#define	futimesat(fd, p, tv)		futimesatl(fd, p, tv)
#define	utimensat(fd, p, ts, f)		utimensatl(fd, p, ts, f)
#endif

/* lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#define	symlink(d, p)			symlinkl(d, p)
#define	readlink(n, b, c)		readlinkl(n, b, c)
#define	lchown(n, uid, gid)		lchownl(n, uid, gid)
#endif

/* mkstemp */
#define	mktemp(t)			mktempl(t)
#define	mkdtemp(t)			mkdtempl(t)

/* stdlib */
#define	opendir(d)			opendirl(d)

/* linux */
#ifdef HAVE_FGETXATTR
#define	getxattr(p, n, v, s)		getxattrl(p, n, v, s)
#define	lgetxattr(p, n, v, s)		lgetxattrl(p, n, v, s)
#define	setxattr(p, n, v, s, f)		setxattrl(p, n, v, s, f)
#define	lsetxattr(p, n, v, s, f)	lsetxattrl(p, n, v, s, f)
#endif

/* 64 */
#if !defined(_LP64)
#define	open64(p, f, ...)		openl64(p, f, ##__VA_ARGS__)
#define	creat64(p, m)			creatl64(p, m)
#define	stat64(p, s)   			statl64(p, s)

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
#define	openat64(fd, p, f, ...)		openatl64(fd, p, f, ##__VA_ARGS__)
#ifdef HAVE_ATTROPEN
#define	attropen64(p, ap, f, ...)	attropenl64(p, ap, f, ##__VA_ARGS__)
#endif
#define	fstatat64(fd, p, s, f)		fstatatl64(fd, p, s, f)
#endif

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
#define	lstat64(p, s)			lstatl64(p, s)
#endif

/* 64 mkstemp*/
#define	mkstemp64(t)			mkstempl64(t)
#define	mkstemps64(t, l)		mkstempsl64(t, l)
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
#define	mkostemp64(t, f)		mkostempl64(t, f)
#define	mkostemps64(t, l, f)		mkostempsl64(t, l, f)
#endif
#endif /* _LP64_ */
#endif /* __PRAGMA_REDEFINE_EXTNAME */

#include "longpath.h"

#endif /* _LONGPATH_REDEFINE_IMPL_H */
