/*
 * table file with functions for which a *64 variant can or cannot exist,
 * depending upon the compilation environment (see lf64 etc)
 *
 * using seperate FUNCTION and FUNCTION64 macros because the 64 variants can
 * have a different signature (read: stat64)
 */

FUNCTION(int, open, const char *, int, ...)
FUNCTION(int, creat, const char *, mode_t)
FUNCTION(int, stat, const char *, struct stat *)
FUNCTION64(int, open, const char *, int, ...)
FUNCTION64(int, creat, const char *, mode_t)
FUNCTION64(int, stat, const char *, struct stat64 *)

/* 64 at funcs */
#if defined(__EXTENSIONS__) || !defined(__XOPEN_OR_POSIX) ||	\
	defined(_ATFILE_SOURCE)
FUNCTION(int, openat, int, const char *, int, ...)
FUNCTION64(int, openat, int, const char *, int, ...)
# ifdef HAVE_ATTROPEN
FUNCTION(int, attropen, const char *, const char *, int, ...)
FUNCTION64(int, attropen, const char *, const char *, int, ...)
# endif
FUNCTION(int, fstatat, int, const char *, struct stat *, int)
FUNCTION64(int, fstatat, int, const char *, struct stat64 *, int)
#endif /* __EXTENSIONS ... */

/* 64 lstat */
#if !defined(__XOPEN_OR_POSIX) || defined(_XPG_2) || defined(__EXTENSIONS__)
FUNCTION64(int, lstat, const char *, struct stat64 *)
FUNCTION(int, lstat, const char *, struct stat *)
#endif /* __XOPEN... */

/* 64 mkstemp*/
FUNCTION(int,   mkstemp, char *)
FUNCTION(int,   mkstemps, char *, int)
FUNCTION64(int, mkstemp, char *)
FUNCTION64(int, mkstemps, char *, int)
/* 64 mkstemp GNU*/
#if defined(_GNU_SOURCE)
#if defined(HAVE_MKOSTEMP)
FUNCTION64(int, mkostemp, char *, int)
FUNCTION(int,   mkostemp, char *, int)
#endif
#if defined(HAVE_MKOSTEMPS)
FUNCTION(int,   mkostemps, char *, int, int)
FUNCTION64(int, mkostemps, char *, int, int)
#endif
#endif /* GNU_SOURCE */

