/*-
 * test code for wrapper library for long paths exceeding PATH_MAX
 * 
 * Copyright (c) 2013 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Nils Goroll <nils.goroll@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * use this tool with some long path files
 *
mkdir /tmp/test
cd /tmp/test
for i in {,1,2,3}{0..9} ; do
  p=`printf "%0255d" $i`;
  mkdir $p && cd $p;
  echo REALLYLONGPATHTEST >file.$i
  date >>file.$i
done
cd /
find /tmp/test -name file\*

test:

find /tmp/test -name file\* | gxargs pfexec truss -o /tmp/tr -f src/longpath_unsafe_test
find /tmp/test -name file\* | gxargs pfexec truss -o /tmp/tr -f src/longpath_test
*/

// for mko*temp*
#define _GNU_SOURCE

#include <longpath/longpath.h>
#include <stdlib.h>
#include <alloca.h>
#ifdef HAVE_SYS_MKDEV_H
#include <sys/mkdev.h>
#else
// linux has makedev in sys/types.h but wants this define
#undef  _BSD_SOURCE
#define _BSD_SOURCE
#endif
#include <sys/types.h>
#include <assert.h>

// XXX conditional includes incomplete
#ifdef HAVE_STRING_H
#include <string.h>
#endif
#ifdef HAVE_LIMITS_H
#include <limits.h>
#endif
#ifdef HAVE_ERRNO_H
#include <errno.h>
#endif
#ifdef HAVE_STDIO_H
#include <stdio.h>
#endif
// open
#ifdef HAVE_SYS_VARARGS_H
#include <sys/varargs.h>
#endif
// opendir
#ifdef HAVE_DIRENT_H
#include <dirent.h>
#endif

// linux
#ifndef S_IAMB
#define S_IAMB (S_ISUID|S_ISGID|S_ISVTX|S_IRWXU|S_IRWXG|S_IRWXO)
#endif

#define _ok(badval, funcn, ...) if (funcn(__VA_ARGS__) == (badval)) {	\
		perror(#funcn);						\
		assert(! #funcn);					\
	}
#define ok(funcn, ...)		_ok(-1  , funcn, __VA_ARGS__)
#define ok_ptr(funcn, ...)	_ok(NULL, funcn, __VA_ARGS__)

#define _nok(badval, e, funcn, ...) if (funcn(__VA_ARGS__) != (badval)) { \
		assert(! "should fail: " #funcn);			\
	} else if (errno != e) {					\
		perror(#funcn " bad errno");				\
		assert(! "expected " #e);				\
	}
#define nok(e, funcn, ...)	_nok(-1  , e, funcn, __VA_ARGS__)
#define nok_ptr(e, funcn, ...)	_nok(NULL, e, funcn, __VA_ARGS__)

#define canskip(p, funcn, ...) if (p) {					\
		funcn(__VA_ARGS__);					\
	} else {							\
		fprintf(stderr, "warn: can't test %s at line %d\n",	\
		#funcn, __LINE__);					\
	}


#define check(funcn, v) if((v) == -1) {			\
		perror(#funcn);				\
		assert(! #funcn);			\
	}

#define otherpath(argc, argv, i) (		\
	((i + 1) < argc) ? argv[i + 1]		\
	: ((i > 1) ? argv[i - 1] : NULL))

#undef  strdupa
#define strdupa(s)				\
	strcpy(alloca(strlen(s) + 1), s)

static char *
_strconcata(char *spc, const char *s1, const char *s2) {
	char *r = spc;
	assert (r);
	strcpy(r, s1);
	strcat(r, s2);
	assert (r);
	return r;
}

static char *
_strconcata3(char *spc, const char *s1, const char *s2, const char *s3) {
	char *r = spc;
	assert (r);
	strcpy(r, s1);
	strcat(r, s2);
	strcat(r, s3);
	assert (r);
	return r;
}

#define strconcata(s1, s2)						\
	_strconcata(alloca(strlen(s1) + strlen(s2) + 1), s1, s2)
#define strconcata3(s1, s2, s3)						\
	_strconcata3(alloca(strlen(s1) + strlen(s2) + strlen(s3) + 1), s1, s2, s3)

static void
test_file(const char *path, const char *path2) {
	const char	*linkpath;
	struct stat	tstat;
	int		n;
	const int	sz	= 1024;
	char		buf[sz];

	linkpath = strconcata(path2 ? path2 : path, ".link");

	assert(linkpath);
	assert(*linkpath);
	/* TEST linkl */
	ok(linkl, path, linkpath);
	/* TEST unlinkl */
	ok(unlinkl, linkpath);

	/* TEST symlinkl */
	ok(symlinkl, strrchr(path, '/') + 1, linkpath);

	/* TEST readlinkl */
	n = readlinkl(linkpath, buf, sz);
	check(readlinkl, n);
	buf[n] = '\0';
	printf("readlink out: %s\n", buf);

	/* TEST lchownl */
	ok(lchownl, path, 2, 3);

	/* TEST lstatl */
	ok(lstatl, path, &tstat);
	assert(tstat.st_uid == 2);
	assert(tstat.st_gid == 3);

	ok(unlinkl, linkpath);

	/* TEST renamel (file) */
	ok(renamel, path, linkpath);
	ok(renamel, linkpath, path);
}

static void
test_dir(const char *path, const char *path2) {
	char		*dp1,  *dp2;
	const char	*cdp1, *cdp2;

	cdp1 = dp1 = strconcata(path,			".dirpath_1_XXXXXX");
	cdp2 = dp2 = strconcata(path2 ? path2 : path,	".dirpath_2_XXXXXX");

	/* TEST mkdirl */
	ok(mkdirl, cdp1, 0700);

#if 0
	/* neg test dir unlink */
	nok(EINVAL, unlinkl, cdp1);
#endif

	/* TEST renamel (dir) */
	ok(renamel, cdp1, cdp2);
	ok(renamel, cdp2, cdp1);

	/* TEST rmdirl */
	ok(rmdirl, cdp1);

	cdp1 = cdp2 = NULL;

	/* TEST mkdtempl */
	ok_ptr(mkdtempl, dp1);
	assert(dp1[strlen(dp1) - 1] != 'X');
	nok_ptr(EEXIST, mkdtempl, dp1);
	ok(rmdirl, dp1);
}

static void
test_mktempl(const char *path) {
	char	*c1, *c2;
	int	fd;

	c1 = strconcata(path, ".mktempl_XXXXXX");
	c2 = strdupa(c1);

	/* TEST mktempl */
	ok_ptr(mktempl, c1);
	assert(strcmp(c1, c2));

	fd = openl(c1, O_CREAT|O_EXCL|O_RDWR, 0600);
	check(openl, fd);
	ok_ptr(mktempl, c2);
	assert(strcmp(c1, c2));

	ok(close, fd);
	ok(unlinkl, c1);
}


#define t_mkXstempXl(fname, ...)					\
static void								\
test_ ## fname(const char *path, const char *templsuff) {		\
       	char		*c1, *c2;					\
       	int		fd;						\
       	struct stat	tstat;						\
									\
	c1 = strconcata3(path, "." #fname "_XXXXXX", templsuff);	\
	c2 = strdupa(c1);						\
									\
	fd = fname(c1, ##__VA_ARGS__);					\
	check(fname, fd);						\
	assert(strcmp(c1, c2));						\
	ok(lstatl, c1, &tstat);					\
	ok(close, fd);							\
									\
	fd = fname(c2, ##__VA_ARGS__);					\
	check(fname, fd);						\
	assert(strcmp(c1, c2));						\
	ok(lstatl, c2, &tstat);		       			\
	ok(close, fd);							\
									\
	ok(unlinkl, c1);						\
	ok(unlinkl, c2);						\
}

t_mkXstempXl(mkstempl)
t_mkXstempXl(mkstempsl, strlen(templsuff))
#if defined(HAVE_MKOSTEMP)
t_mkXstempXl(mkostempl, O_SYNC)
#endif
#if defined(HAVE_MKOSTEMPL)
t_mkXstempXl(mkostempsl, strlen(templsuff), O_SYNC)
#endif

static void
test_mkXstempXl(const char *path) {
	/* TEST mkstempl */
	/* TEST mkstempsl */
	/* TEST mkostempl */
	/* TEST mkostempsl */
	test_mkstempl(path, "");
	test_mkstempsl(path, "suffix123");
#if defined(HAVE_MKOSTEMP)
	test_mkostempl(path, "");
#endif
#if defined(HAVE_MKOSTEMPL)
	test_mkostempsl(path, "suffix123");
#endif
}

static void
test_mkfifol(const char *path) {
	char		*c1;
	struct stat	tstat;

	c1 = strconcata(path, ".fifo");

	ok(mkfifol, c1, 0700);
	ok(lstatl, c1, &tstat);
	assert(tstat.st_mode & S_IFIFO);

	ok(unlinkl, c1);
}

static void
test_mknodl(const char *path) {
#ifdef SKIP_TEST_MKNOD
	(void)path;
#else
	char		*c1;
	struct stat	tstat;

	c1 = strconcata(path, ".nod");

	ok(mknodl, c1, 0700|S_IFCHR, makedev(37, 0));
	ok(lstatl, c1, &tstat);
	assert(tstat.st_mode & S_IFCHR);

	ok(unlinkl, c1);
#endif
}

#define tv_eq(a, b)						\
	((a.tv_sec == b.tv_sec) && (a.tv_nsec == b.tv_nsec))

int
main(int argc, char *argv[]) {
	int		fd;
	struct stat	tstat, tostat;
	int		i, n;
	char		*path, *otherpath;
	const int	sz	= 1024;
	char		buf[sz];

	if (argc < 2)
		return (EINVAL);

	for (i = 1; i < argc; i++) {
		path = argv[i];
		otherpath = otherpath(argc, argv, i);

		if (otherpath)
			printf("length path, otherpath:\t%8zu\t%8zu\n", strlen(path), strlen(otherpath));
#if 0
		printf("path: %s\n", path);
		printf("otherpath: %s\n", otherpath);
#endif
		/* TEST openl */
		errno = 0;

		fd = openl(path, O_RDONLY, 0);
		check(openl, fd);

		// no result ouput
		ok(fstat, fd, &tstat);

		// print the first bytes up to sz size
		// ignore any errors
		while ((n = read(fd, buf, sz)) != -1) {
			if (write(1, buf, n) != n)
				break;
			if (n < sz)
				break;
		}

		ok(close, fd);

		/* TEST statl */
		ok(statl, path, &tostat);

		/* TEST chownl */
		ok(chownl, path, 2, 3);
		/* TEST chmodl */
		ok(chmodl, path, 0452);
		ok(statl, path, &tstat);
		assert(tstat.st_uid == 2);
		assert(tstat.st_gid == 3);
		assert((tstat.st_mode & S_IAMB) == 0452);

		/* TEST utimesl */
		ok(utimesl, path, NULL);
		ok(statl, path, &tstat);
		assert(! tv_eq(tstat.st_atim, tostat.st_atim));
		assert(! tv_eq(tstat.st_mtim, tostat.st_mtim));

		/* XXX HERE */

		/* TEST utimel */
		ok(utimel, path, NULL);

		/* chown back */
		ok(chownl, path, tostat.st_uid, tostat.st_gid);
		/* chmod back */
		ok(chmodl, path, tostat.st_mode & S_IAMB);
		/* check everything back ? */
		ok(statl, path, &tstat);
		assert(tstat.st_uid == tostat.st_uid);
		assert(tstat.st_gid == tostat.st_gid);
		assert(tstat.st_mode == tostat.st_mode);

		test_file(path, otherpath);
		test_dir(path, otherpath);
		test_mktempl(path);
		test_mkXstempXl(path);
		test_mknodl(path);
		test_mkfifol(path);
	}
	return 0;
}

